/*
 * Copyright (C) 2020 CERN (www.cern.ch)
 *
 * Author: Miguel Jimenez Lopez <miguel.jimenez@sevensols.com>
 *
 * Released to the public domain as sample code to be customized.
 *
 * This work is part of the White Rabbit project, a research effort led
 * by CERN, the European Institute for Nuclear Research.
 */

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>

#include "log-device.h"
#include "file-log-private.h"

static struct log_device_iface _file_log_iface =
{
	.init_log_device = init_file_log_device,
	.send_to_log_device = send_to_file_log_device,
	.clear_log_device = NULL,
	.deinit_log_device = deinit_file_log_device,
};
static const log_device_interface file_log_iface = &_file_log_iface;

log_device create_file_log_device(const char *path)
{
	return create_log_device((void *)path, file_log_iface);
}

static void init_file_log_device(log_device dev)
{
	struct file_log_private *priv;

	priv = calloc(1, sizeof(*priv));
	priv->path = dev->private;
	priv->fp = fopen(priv->path, "w+");

	dev->private = priv;
}

static void send_to_file_log_device(log_device dev, const char *msg)
{
	struct file_log_private *priv;

	if(!check_file_log_priv(dev)) {
		priv = get_private_info(dev);
		fputs(msg, priv->fp);
		fputs("\n", priv->fp);
	}
}

static void deinit_file_log_device(log_device dev)
{
	struct file_log_private *priv;

	if(dev->private) {
		priv = get_private_info(dev);
		fclose(priv->fp);
		free(dev->private);
	}
}

static int check_file_log_priv(log_device dev)
{
	return (dev->private != NULL) ? 0 : 1;
}

static struct file_log_private * get_private_info(log_device dev)
{
	return (struct file_log_private *)dev->private;
}
