/*
 * Copyright (C) 2020 CERN (www.cern.ch)
 *
 * Author: Miguel Jimenez Lopez <miguel.jimenez@sevensols.com>
 *
 * Released to the public domain as sample code to be customized.
 *
 * This work is part of the White Rabbit project, a research effort led
 * by CERN, the European Institute for Nuclear Research.
 */

#ifndef __FMC_DIO_DEVICE_PRIVATE_H__
#define __FMC_DIO_DEVICE_PRIVATE_H__

#include "usr-timestamp.h"
#include "../../kernel/fmc-dio.h"

#define MAX_TMP_BUF 1024
#define FMC_DIO_N_CH 5
#define NS_IN_A_SEC 1000000000
#define MAX_FMC_DIO_PATH 100

struct fmc_dio_dev {
	char dev_name[MAX_FMC_DIO_PATH];
	int fd;

	log_device *logs;
	unsigned int n_logs;
	int enabled_log;

	unsigned int total_n_ts;
	unsigned int partial_n_ts;

	struct wr_dio_cmd cmd;
};

static const fmc_dio_device BAD_FMC_DEVICE = NULL;

static int check_fmc_dio_device(fmc_dio_device dev);

static int enable_fmc_dio_device_hw_irq(fmc_dio_device dev,
					int ch, int en);
static int setup_fmc_dio_device_hw_irq(fmc_dio_device dev,
				unsigned long period,
				long count);

static int get_hw_ts_from_fmc_dio_device(fmc_dio_device dev, int ch,
					struct usr_timestamp **ts, unsigned int *nts);
static int get_kernel_leaps_info(fmc_dio_device dev);

static void log_msg(fmc_dio_device dev, const char *msg);
static void LOG(fmc_dio_device dev, const char *fmt, ...);

#endif
