/*
 * Copyright (C) 2020 CERN (www.cern.ch)
 *
 * Author: Miguel Jimenez Lopez <miguel.jimenez@sevensols.com>
 *
 * Released to the public domain as sample code to be customized.
 *
 * This work is part of the White Rabbit project, a research effort led
 * by CERN, the European Institute for Nuclear Research.
 */

#ifndef __FMC_DIO_DEVICE_H__
#define __FMC_DIO_DEVICE_H__

#include "log-device.h"
#include "usr-timestamp.h"

typedef struct fmc_dio_dev * fmc_dio_device;

fmc_dio_device create_fmc_dio_device(char *name);
int open_fmc_dio_device(fmc_dio_device dev);
void close_fmc_dio_device(fmc_dio_device dev);

void attach_log_devices_to_fmc_dio_device(fmc_dio_device dev,
				log_device *log_devs,
				unsigned int n_log_devs);
int enable_log_for_fmc_dio_device(fmc_dio_device dev);
int disable_log_for_fmc_dio_device(fmc_dio_device dev);

int enable_fmc_dio_device_irq(fmc_dio_device dev, int ch);
int enable_fmc_dio_device_all_irq(fmc_dio_device dev);
int disable_fmc_dio_device_irq(fmc_dio_device dev, int ch);
int disable_fmc_dio_device_all_irq(fmc_dio_device dev);
int setup_fmc_dio_device_irq(fmc_dio_device dev,
			unsigned long period,
			long count);

int get_tai_ts_from_fmc_dio_device(fmc_dio_device dev, int ch,
				struct usr_timestamp **ts, unsigned int *nts);
int get_utc_ts_from_fmc_dio_device(fmc_dio_device dev, int ch,
				struct usr_timestamp **ts, unsigned int *nts);

unsigned int get_total_n_timestamps(fmc_dio_device dev);
unsigned int get_partial_n_timestamps(fmc_dio_device dev);
void reset_partial_n_timestamps(fmc_dio_device dev);

#endif
