/*
 * Copyright (C) 2020 CERN (www.cern.ch)
 *
 * Author: Miguel Jimenez Lopez <miguel.jimenez@sevensols.com>
 *
 * Released to the public domain as sample code to be customized.
 *
 * This work is part of the White Rabbit project, a research effort led
 * by CERN, the European Institute for Nuclear Research.
 */

#include <stdlib.h>

#include "log-device.h"
#include "log-device-private.h"

log_device create_log_device(void *args, log_device_interface interface)
{
	log_device dev = calloc(1, sizeof(struct log_dev));

	if(dev != BAD_LOG_DEVICE && interface != NULL) {
		dev->private = args;
		dev->ops = interface;
		if(dev->ops->init_log_device)
			dev->ops->init_log_device(dev);
	}

	return dev;
}

void send_to_log_device(log_device dev, const char *msg)
{
	if(check_log_device_interface(dev))
		return;

	dev->ops->send_to_log_device(dev, msg);
}

void clear_log_device(log_device dev)
{
	if(check_log_device_interface(dev))
		return;

	if(dev->ops->clear_log_device)
		dev->ops->clear_log_device(dev);
}

void destroy_log_device(log_device dev)
{
	if(check_log_device_interface(dev))
		return;

	if(dev->ops->deinit_log_device)
		dev->ops->deinit_log_device(dev);
	free(dev);
}

static int check_log_device(log_device dev)
{
	return (dev == BAD_LOG_DEVICE) ? 1 : 0;
}

static int check_log_device_interface(log_device dev)
{
	if(check_log_device(dev))
		return 1;

	if(!dev->ops)
		return 1;
	else
		return 0;
}

