/*
 * Copyright (C) 2020 CERN (www.cern.ch)
 *
 * Author: Miguel Jimenez Lopez <miguel.jimenez@sevensols.com>
 *
 * Released to the public domain as sample code to be customized.
 *
 * This work is part of the White Rabbit project, a research effort led
 * by CERN, the European Institute for Nuclear Research.
 */

#include <stddef.h>
#include <stdio.h>

#include "log-device.h"
#include "printf-log-private.h"

static struct log_device_iface _printf_log_iface =
{
	.init_log_device = NULL,
	.send_to_log_device = send_to_printf_log_device,
	.deinit_log_device = NULL,
	.clear_log_device = clear_printf_log_device,
};
static const log_device_interface printf_log_iface = &_printf_log_iface;

static void send_to_printf_log_device(log_device dev, const char *msg)
{
	printf("%s\n", msg);
}

static void clear_printf_log_device(log_device dev)
{
	printf("\e[1;1H\e[2J");
}

log_device create_printf_log_device(void)
{
	return create_log_device(NULL, printf_log_iface);
}
