/*
 * Copyright (C) 2020 CERN (www.cern.ch)
 *
 * Author: Miguel Jimenez Lopez <miguel.jimenez@sevensols.com>
 *
 * Released to the public domain as sample code to be customized.
 *
 * This work is part of the White Rabbit project, a research effort led
 * by CERN, the European Institute for Nuclear Research.
 */

#ifndef __STATS_ENGINE_PRIVATE_H__
#define __STATS_ENGINE_PRIVATE_H__

#include "usr-timestamp.h"

#define MAX_TS 1000
#define NS_IN_A_SEC 1000000000
#define MAX_TMP_BUF 1024
#define METRICS_UNIT "s"

struct ts_stats {
	double mean;
	double std;
	unsigned int ts_per_second;
	unsigned int ts_total;
};

struct _stats_engine {
	struct usr_timestamp usr_ts[MAX_TS];
	struct timespec sys_ts[MAX_TS];
	double diff_ts[MAX_TS];
	struct ts_stats stats;
	unsigned int n_ts;
	unsigned int current_ts;
	unsigned int next_ts;
	unsigned int n_iter;

	log_device *logs;
	unsigned int n_logs;
	int enabled_log;
};

const stats_engine BAD_STATS_ENGINE = NULL;

static void update_n_ts_stats_engine(stats_engine engine);
static void update_index_stats_engine(unsigned int *index);

static void log_stats_engine(stats_engine engine, int verbose);
static int check_log_configuration(stats_engine engine);
static void log_msg(stats_engine engine,
		const char *msg);
static void log_clear(stats_engine engine);
static void LOG(stats_engine engine, const char *fmt, ...);

static int timespec_subtract(struct timespec *result,
			struct timespec *x,
			struct timespec *y);
static double compute_mean(const double *values,
			const unsigned int n_values);
static double compute_stdev(const double *values,
			const unsigned int n_values,
			const double mean);
static int compute_ts_per_second(const int sec,
				const struct usr_timestamp *ts,
				const unsigned int n_ts);

#endif
