/*
 * Copyright (C) 2019 CERN (www.cern.ch)
 * Author: Miguel Jimenez Lopez <miguel.jimenez@sevensols.com>
 *
 * Released according to the GNU GPL, version 2 or any later version.
 *
 * This work is part of the White Rabbit project, a research effort led
 * by CERN, the European Institute for Nuclear Research.
 */

#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/slab.h>
#include <linux/fmc.h>

#include "fmc-dio.h"

static int fmc_dio_gpio_input(struct gpio_chip *chip, unsigned offset)
{
	return -EAGAIN;
}

static int fmc_dio_gpio_output(struct gpio_chip *chip, unsigned offset, int value)
{
	return -EAGAIN;
}

int fmc_dio_gpio_get(struct gpio_chip *chip, unsigned offset)
{
	return -EAGAIN;
}

void fmc_dio_gpio_set(struct gpio_chip *chip, unsigned offset, int value)
{
	return;
}

static const char *fmc_dio_gpio_names[] = {
	"dire", "fare", "baciare", "lettera", "testamento"
};

static struct gpio_chip fmc_dio_gpio_template = {
	.label = "fmc-dio",
	.owner = THIS_MODULE,
	/* FIXME: request, free, for multi-function operation */
	.direction_input = fmc_dio_gpio_input,
	.direction_output = fmc_dio_gpio_output,
	.get = fmc_dio_gpio_get,
	.set = fmc_dio_gpio_set,
	.base = -1, /* request dynamic */
	.ngpio = 5,
	.names = fmc_dio_gpio_names,
};

int fmc_dio_gpio_init(struct fmc_dio *dev)
{
	struct fmc_device *fmc = dev->fmc;
	struct gpio_chip *gc;
	int ret;

	gc = devm_kzalloc(&dev->fmc->dev, sizeof(*gc), GFP_KERNEL);
	if (!gc)
		return -ENOMEM;
	*gc = fmc_dio_gpio_template;
#if LINUX_VERSION_CODE < KERNEL_VERSION(4,5,0)
	gc->dev = &fmc->dev;
#else
	gc->parent = &fmc->dev;
#endif

	ret = gpiochip_add(gc);
	if (ret < 0)
		goto out_free;
	dev->gc = gc;

	/* FIXME: program the DAC for each port (sysfs attributes?) */
	return 0;

out_free:
	kfree(gc);
	return ret;
}

void fmc_dio_gpio_exit(struct fmc_dio *dev)
{
	struct gpio_chip *gc = dev->gc;

#if LINUX_VERSION_CODE > KERNEL_VERSION(3,17,0)
	gpiochip_remove(gc);
#else
	int ret;

	ret = gpiochip_remove(gc);
	if (ret)
		dev_err(&dio->pdev->dev, "DANGER %i! gpio chip can't be removed\n",
			ret);
#endif
	return;
}
