/*
 * Copyright (C) 2019 CERN (www.cern.ch)
 * Author: Miguel Jimenez Lopez <miguel.jimenez@sevensols.com>
 *
 * Released according to the GNU GPL, version 2 or any later version.
 *
 * This work is part of the White Rabbit project, a research effort led
 * by CERN, the European Institute for Nuclear Research.
 */

#include <linux/module.h>
#include <linux/version.h>

#include "fmc-dio.h"

static int fmc_dio_mdev_open(struct inode *inode, struct file *file)
{

	struct miscdevice *mdev_ptr = file->private_data;

	// WARN: Update the private_data pointer to point to fmc_dio structure
	file->private_data = container_of(mdev_ptr, struct fmc_dio, mdev);

	return 0;
}

static int fmc_dio_mdev_release(struct inode *inode, struct file *file)
{
	return 0;
}

static ssize_t fmc_dio_mdev_write(struct file *file, const char __user *buf,
		size_t count, loff_t *offp)
{
	return 0;
}

static ssize_t fmc_dio_mdev_read (struct file *file, char __user *buf,
		size_t count, loff_t *offp)
{
	return 0;
}

static long fmc_dio_mdev_unlocked_ioctl (struct file *file, unsigned int cmd,
		unsigned long arg)
{
	struct fmc_dio *dio = file->private_data;
	
	return fmc_dio_int_ioctl(dio,cmd,arg);
}

static const struct file_operations fmc_dio_mdev_fops = {
	.owner = THIS_MODULE,
	.open = fmc_dio_mdev_open,
	.release = fmc_dio_mdev_release,
	.write  = fmc_dio_mdev_write,
	.read = fmc_dio_mdev_read,
	.unlocked_ioctl = fmc_dio_mdev_unlocked_ioctl,
};

int fmc_dio_mdev_create(struct fmc_dio *dio)
{
	dio->mdev.minor = MISC_DYNAMIC_MINOR;
	dio->mdev.fops = &fmc_dio_mdev_fops;
	dio->mdev.name = dio->name;

	return misc_register(&dio->mdev);
}

void fmc_dio_mdev_destroy(struct fmc_dio *dio)
{
	misc_deregister(&dio->mdev);
}
